/*******************************************************************************
   RoBoard I2C Master-Mode Simple Tool

     This program is for the purpose of demonstrating RoBoard I2C master function. 
     Execute it without arugments to see how to use.

*******************************************************************************/
 
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#if defined(linux)
    #define stricmp  strcasecmp
#endif

#if defined WINCE
    #include <windows.h>
	#define stricmp _stricmp
#endif

#if defined(WINCE) || defined(WIN32)
    #include <roboard_dll.h>  // use the DLL version of RoBoIO lib
    // #include <roboard.h>   // use the static version of RoBoIO lib
#else
    #include <roboard.h>
#endif

void help(char *s) {
    int l = strlen(s);

    for (; l>=0; l--) if ((s[l] == '/') || (s[l] == '\\')) break;
    if (l >= 0) s = &s[l+1];

	printf(
	    "Usage                                                                \n"
	    "=======                                                              \n"
        "%s [ver] [addr] [bps] [-w byte1 byte2 ...] [-r count]                \n"
	    "                                                                     \n"
	    "   [ver] (RoBoard version)                                           \n"
	    "      RB050   : RoBoard RB-050                                       \n"
	    "      RB100   : RoBoard RB-100                                       \n"
	    "      RB100RD : RoBoard RB-100RD                                     \n"
	    "      RB110   : RoBoard RB-110                                       \n"
	    "                                                                     \n"
        "   [addr]: I2C device address (reserved I2C address is not allowed)  \n"
        "      The address is a HEX number (<= 7fh).                          \n"
	    "                                                                     \n"
        "   [bps]: I2C clock speed                                            \n"
        "      -1: 10K bps     -2: 50K bps     -3: 100K bps                   \n"
        "      -4: 200K bps    -5: 400K bps                                   \n"
	    "                                                                     \n"
        "    [-w byte1 byte2 ...]: do I2C write                               \n"
        "        write byte1, byte2, ... to I2C device.                       \n"
	    "                                                                     \n"
        "    [-r count]: do I2C read                                          \n"
        "        read [count] bytes from I2C device.                          \n"
        "                                                                     \n"
		"   Note that byte1, byte2, ... should be represented in HEX numbers. \n"
        "                                                                     \n"
        "<Press Enter to continue...>                                         \n"
	    , s
    );
	getchar();

	printf(
        "example                                                          \n"
        "=========                                                        \n"
        "%s RB100 70 -2 -w 00 51                                          \n"
        "    write 0x00 0x51 to the I2C device;                           \n"
        "    I2C device address: 0x70, I2C clock speed: 50K bps           \n"
        "                                                                 \n"
        "%s RB110 32 -3 -r 2                                              \n"
        "    read 2 bytes from the I2C devicel                            \n"
        "    I2C device address: 0x32, I2C clock speed: 100K bps          \n"
        "                                                                 \n"
        "%s RB050 30 -5 -w 02 -r 2                                        \n"
        "    first write 0x02 to the I2C device and then restart to       \n"
        "    read 2 bytes from the I2C device;                            \n"
        "    I2C device address: 0x30, I2C clock speed: 400K bps          \n"
        "                                                                 \n"
        , s, s, s
    );
}

bool StringToHex(char *s, unsigned char *val)
{
    unsigned int tmp = 0;

	if (s[0] == '0')
	{
		if(s[1] == 'x' || s[1] == 'X') s = &s[2];
	}
	
	while(*s != '\0')
	{
		if(*s >= '0' && *s <= '9')
		{
			tmp = tmp * 16 + *s - '0';
		}
		else if(*s >= 'a' && *s <= 'f')
		{
			tmp = tmp * 16 + *s - 'a' + 10;
		}
		else if(*s >= 'A' && *s <= 'F')
		{
			tmp = tmp * 16 + *s - 'A' + 10;
		}
		else
		  return false;

        s++;
	}

    if (tmp > 255) return false;
    
    *val = (unsigned char)tmp;
	return true;
}


bool CheckAddress(unsigned char address)
{
	if(address > 0x7f)
	{
		printf("The address is over the range.\n\n");
		return false;
	}
	switch(address)
	{
		case 0x01:
			printf("The address is reserved for the obsolete C-Bus format.\n\n");
			return false;
		case 0x02:
			printf("The address is reserved for a different bus format.\n\n");
			return false;
		case 0x04:
		case 0x05:
		case 0x06:
		case 0x07:
			printf("The address is reserved for Hs-mode master code.\n\n");
			return false;
		case 0x03:
		case 0x7c:
		case 0x7d:
		case 0x7e:
		case 0x7f:
			printf("The address is reserved for future purposes.\n\n");
			return false;
		case 0x78:
		case 0x79:
		case 0x7a:
		case 0x7b:
			printf("The address is reserved for 10-bit slave addressing.\n\n");
			return false;
		default:
			return true;
	}
}

unsigned char GetAddress(char *s)
{
    unsigned char val = 0;
	if(StringToHex(s,&val) == false)
	{
		printf("ERR: wrong I2C address!!!\n\n");
		exit(1);
    }		
	if(CheckAddress(val) == false)
	{
		printf("ERR: reserved I2C address!!!\n\n");
		exit(1);
	}
    return val;	
}

unsigned long SetI2CSpeed(char *s)
{
	if(stricmp("-1",s) == 0)
	{
		return 10000L;
	}
	else if(stricmp("-2",s) == 0)
	{
		return 50000L;
	}
	else if(stricmp("-3",s) == 0)
	{
		return 100000L;
	}
	else if(stricmp("-4",s) == 0)
	{
		return 200000L;
	}
	else if(stricmp("-5",s) == 0)
	{
		return 400000L;
	}
	else
	{
		printf("ERR: wrong I2C clock speed parameter!!!\n\n");
		exit(1);
	}
}

void GetI2CAction(int argc, char *argv[], unsigned char *wdata, int *wcnt, int *rcnt)
{
	int m = 4, wsize = *wcnt;

    *wcnt = *rcnt = 0;

    if ((m < argc) && (stricmp("-w",argv[m]) == 0))
    {
        for (m=m+1; m<argc; m++)
        {
            if (stricmp("-r",argv[m]) == 0) break;
            
            if (*wcnt >= wsize)
            {
		        printf("ERR: too many bytes to write!!!\n\n");
		        exit(1);
            }
        
            if (StringToHex(argv[m],wdata) == false)
            {
		        printf("ERR: there exists a wrong byte to write!!!\n\n");
		        exit(1);
            }
            
            (*wcnt)++;
            wdata++;
        }
        
        if (*wcnt == 0)
        {
		    printf("ERR: no byte to write!!!\n\n");
		    exit(1);
        }
    }
    
    if ((m < argc) && (stricmp("-r",argv[m]) == 0))
    {
        if (++m < argc) *rcnt = atoi(argv[m++]);
        
        if ((*rcnt <= 0) || (*rcnt > wsize))
        {
		    printf("ERR: wrong number of bytes to read!!!\n\n");
		    exit(1);
        }
    }
    
    if (m < argc)
    {
		printf("ERR: wrong I2C r/w parameters!!!\n\n");
		exit(1);
    }

    return;
}

void SetRBVer(int argc, char* argv[]) {
    if (stricmp(argv[1], "RB050") == 0)
    {
        roboio_SetRBVer(RB_050); return;
    }
    else
    if (stricmp(argv[1], "RB100") == 0)
    {
        roboio_SetRBVer(RB_100); return;
    }
    else
    if (stricmp(argv[1], "RB100b1") == 0)
    {
        roboio_SetRBVer(RB_100b1); return;
    }
    else
    if (stricmp(argv[1], "RB100b2") == 0)
    {
        roboio_SetRBVer(RB_100b2); return;
    }
    else
    if (stricmp(argv[1], "RB100RD") == 0)
    {
        roboio_SetRBVer(RB_100RD); return;
    }
    else
    if (stricmp(argv[1], "RB110") == 0)
    {
        roboio_SetRBVer(RB_110); return;
    }

	printf("ERR: unknown RoBoard version!!!\n\n");
    exit(1);
}


#ifdef WINCE
int _tmain(int argc, _TCHAR* _argv[])
{
	char  argvbuf[50][100] = {'\0'};
	char* argv[50];

	if (argc > 50) argc = 50;
	for (int _argc=0; _argc<argc; _argc++)
	{
		argv[_argc] = &(argvbuf[_argc][0]);
		wcstombs(argv[_argc], _argv[_argc], 100);
	}
#else
int main(int argc, char* argv[])
{
#endif

	unsigned char i2c_address;
	unsigned long i2c_clock;
	unsigned char i2c_data[256];
	int wcnt = 256, rcnt, i;

	if(argc == 1)
	{
		help(argv[0]);
		return 0;
	}

	if(argc < 6)
	{
		printf("ERR: wrong number of parameters!!!\n\n");
		return 1;
	}

	SetRBVer(argc, argv);

	i2c_address = GetAddress(argv[2]);
	i2c_clock   = SetI2CSpeed(argv[3]);
	GetI2CAction(argc, argv, i2c_data, &wcnt, &rcnt);	


	if(i2c_Init(I2CMODE_AUTO, i2c_clock) == false)
	{
        printf("ERR: fail to initialize I2C (%s)!\n", roboio_GetErrMsg());
        return 1;
	}

    if (wcnt != 0)
    {
        printf("Write ");
		for(i=0; i<wcnt; i++) printf("%02Xh ", i2c_data[i]);
		printf("to I2C bus...");

        if (rcnt == 0)
        {
            if (i2c_Send(i2c_address, i2c_data, wcnt) == false)
                printf("fail (%s)\n", roboio_GetErrMsg());
            else
                printf("success\n");
        }
        else
        {
            if (i2c_SensorReadEX(i2c_address, i2c_data, wcnt, i2c_data, rcnt) == false)
                printf("fail (%s)\n", roboio_GetErrMsg());
            else
            {
                printf("success\n\n");
                for(i=0; i<rcnt; i++) printf("Read byte %d = %02Xh\n", i, i2c_data[i]);
            }
        }
    }
    else if (rcnt != 0)
    {
		printf("Read %d bytes from I2C bus...", rcnt);

        if (i2c_Receive(i2c_address, i2c_data, rcnt) == false)
            printf("fail (%s)\n", roboio_GetErrMsg());
        else
        {
            printf("success\n\n");
            for(i=0; i<rcnt; i++) printf("Read byte %d = %02Xh\n", i, i2c_data[i]);
        }
    }

	i2c_Close();
	return 0;
}

