/*******************************************************************************
   RoBoard Simple RC servo Tool

     This program is for the purpose of demonstrating RoBoIO RC Servo lib. 
     Execute it without arguments to see how to use.

*******************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#if defined(linux)
    #define stricmp  strcasecmp
#endif

#if defined WINCE
    #include <windows.h>
	#define stricmp _stricmp
#endif

#if defined(WINCE) || defined(WIN32)
    #include <roboard_dll.h>  // use the DLL version of RoBoIO lib
    // #include <roboard.h>   // use the static version of RoBoIO lib
#else
    #include <roboard.h>
#endif

int rbver = 100;

void help(char* s) {
    int l = strlen(s);

    for (; l>=0; l--) if ((s[l] == '/') || (s[l] == '\\')) break;
    if (l >= 0) s = &s[l+1];

	printf(
	    "Usage                                                              \n"
	    "=======                                                            \n"
	    "%s [ver] [pin] [type] [-r|-p position time (home)]                 \n"
	    "                                                                   \n"
	    "   [ver] (RoBoard version)                                         \n"
	    "      RB050   : RoBoard RB-050                                     \n"
	    "      RB100   : RoBoard RB-100                                     \n"
	    "      RB100RD : RoBoard RB-100RD                                   \n"
	    "      RB110   : RoBoard RB-110                                     \n"
	    "                                                                   \n"
	    "   [pin] :                                                         \n"
	    "      S1 ~ S24 (for RB-100/RB-100RD)                               \n"
	    "      S1 ~ S16 (for RB-110/RB-050)                                 \n"
	    "                                                                   \n"
	    "   [type] (servo type):                                            \n"
	    "      KRS78X:    KONDO KRS786/KRS788 servo                         \n"
	    "      KRS4014:   KONDO KRS4014 servo                               \n"
	    "      KRS4024:   KONDO KRS4024 servo                               \n"
	    "      HSR8498:   HiTEC HSR8498 servo                               \n"
	    "      S3003:     Futaba S3003 servo                                \n"
	    "      SYS214050: Shayang Ye SYS-214050 servo                       \n"
	    "      ...                                                          \n"
	    "                                                                   \n"
	    "(Press Enter to continue...)                                       \n"
        ,s
    );
	getchar();

	printf(
	    "%s [ver] [pin] [type] [-r|-p position time (home)]                     \n"
	    "                                                                       \n"
	    "      ...                                                              \n"
	    "      MG995:     TowerPro MG995 servo                                  \n"
	    "      MG996:     TowerPro MG996 servo                                  \n"
	    "      RS0263:    DMP RS-0263 servo                                     \n"
	    "      RS1270:    DMP RS-1270 servo                                     \n"
        "      unknown:   other general non-feedback PWM servos                 \n"
	    "                                                                       \n"
	    "   [-r|-p] (demo function):                                            \n"
        "      -r: read servo positions (only work for KONDO/HiTEC servo)       \n"
        "      -p: move servo to the target position from the home position     \n"
	    "                                                                       \n"
        "   [position]:                                                         \n"
        "      target position (in microsecond)                                 \n"
	    "                                                                       \n"
        "   [time]:                                                             \n"
        "      time for moving to the target position (in millisecond)          \n"
	    "                                                                       \n"
        "   [home]:                                                             \n"
        "      initial position of the servo (can ignore for KONDO/HiTEC servo) \n"
	    "                                                                       \n"
	    "(Press Enter to continue...)                                           \n"
        ,s
    );
	getchar();

	printf(
        "example                                                           \n"
        "=========                                                         \n"
        "%s RB100 S1 KRS78X -r                                             \n"
        "    read the position of KRS786 on PWM/GPIO pin S1.               \n"
        "                                                                  \n"
        "%s RB100 S3 HRS8498 -p 1500 500                                   \n"
        "    move HiTEC HSR8498 on pin S3 to 1500us position in time 0.5s. \n"
        "                                                                  \n"
        "%s RB110 S5 S3003 -p 2000 600 1500                                \n"
        "    move Futaba S3003 on pin S5 from 1500us to 2000us positions   \n"
        "    in time 0.6s.                                                 \n"
        "                                                                  \n"
        , s, s, s
    );

}


bool IsFeedback(int svtype) {
    switch (svtype)
    {
        case RCSERVO_KONDO_KRS78X:
        case RCSERVO_KONDO_KRS4014:
        case RCSERVO_KONDO_KRS4024:
        case RCSERVO_HITEC_HSR8498:
            return true;
    }

    return false;
}

int GetServoType(char *s)
{
	if (stricmp("KRS78X", s) == 0)    return RCSERVO_KONDO_KRS78X;
	else
    if (stricmp("KRS4014", s) == 0)   return RCSERVO_KONDO_KRS4014;
	else
    if (stricmp("KRS4024", s) == 0)   return RCSERVO_KONDO_KRS4024;
	else
    if (stricmp("HSR8498", s) == 0)   return RCSERVO_HITEC_HSR8498;
	else
    if (stricmp("S3003", s) == 0)     return RCSERVO_FUTABA_S3003;
	else
    if (stricmp("SYS214050", s) == 0) return RCSERVO_SHAYYE_SYS214050;
	else
    if (stricmp("MG995", s) == 0)     return RCSERVO_TOWERPRO_MG995;
	else
    if (stricmp("MG996", s) == 0)     return RCSERVO_TOWERPRO_MG996;
	else
    if (stricmp("RS0263", s) == 0)    return RCSERVO_DMP_RS0263;
	else
    if (stricmp("RS1270", s) == 0)    return RCSERVO_DMP_RS1270;
	else
    if (stricmp("unknown", s) == 0)   return RCSERVO_SERVO_DEFAULT_NOFB;
	else
	{
		printf("ERR: wrong servo type!!!\n\n");
		exit(1);
	}
}

#define SERVO_READ			 1
#define SERVO_PLAY			 2
int GetServoAction(char *s)
{
	if (stricmp("-r", s) == 0)
		return SERVO_READ;
	else
    if (strcmp("-p", s) == 0)
		return SERVO_PLAY;
	else
	{
		printf("ERR: wrong servo demo funtion!!!\n\n");
		exit(1);
	}
}

int GetChannel(char* str) {
	int val, maxch;

    if ((str[0] == 'S') || (str[0] == 's'))
    {

        val = atoi(&str[1]);
        if ((rbver == 100) || (rbver == 102) || (rbver == 103)) maxch = 24; else if (rbver == 101) maxch = 32; else maxch = 16;
        if ((val >= 1) && (val <= maxch)) return val-1;
	}

    printf("ERR: wrong PWM/GPIO pin!!!\n\n");
    exit(1);
}

void SetRBVer(int argc, char* argv[]) {
    if (stricmp(argv[1], "RB050") == 0)
    {
        roboio_SetRBVer(RB_050); rbver = 50; return;
    }
    else
    if (stricmp(argv[1], "RB100") == 0)
    {
        roboio_SetRBVer(RB_100); rbver = 100; return;
    }
    else
    if (stricmp(argv[1], "RB100b1") == 0)
    {
        roboio_SetRBVer(RB_100b1); rbver = 101; return;
    }
    else
    if (stricmp(argv[1], "RB100b2") == 0)
    {
        roboio_SetRBVer(RB_100b2); rbver = 102; return;
    }
    else
    if (stricmp(argv[1], "RB100RD") == 0)
    {
        roboio_SetRBVer(RB_100RD); rbver = 103; return;
    }
    else
    if (stricmp(argv[1], "RB110") == 0)
    {
        roboio_SetRBVer(RB_110); rbver = 110; return;
    }

	printf("ERR: unknown RoBoard version!!!\n\n");
    exit(1);
}


#ifdef WINCE
int _tmain(int argc, _TCHAR* _argv[])
{
	char  argvbuf[50][100] = {'\0'};
	char* argv[50];

	if (argc > 50) argc = 50; 
	for (int _argc = 0; _argc < argc; _argc++)
	{
		argv[_argc] = &(argvbuf[_argc][0]);
		wcstombs(argv[_argc], _argv[_argc], 100);
	}
#else
int main(int argc, char* argv[])
{
#endif

	int channel, servo_type, servo_demo;
	unsigned long servo_position, servo_time, home[32] = {0L};

	if(argc == 1)
	{
		help(argv[0]);
		return 0;
	}

	if(argc < 5 || argc > 8)
	{
        printf("ERR: wrong number of parameters!\n\n");
		exit(1);
	}

	SetRBVer(argc, argv);

	channel    = GetChannel(argv[2]);
	servo_type = GetServoType(argv[3]);
	servo_demo = GetServoAction(argv[4]);
    
	if (servo_demo == SERVO_PLAY)
	{
		if(argc < 7)
        {
            printf("ERR: wrong number of parameters!!!\n\n");
		    exit(1);
		}

		servo_position = atol(argv[5]);
		servo_time     = atol(argv[6]);

		if ((servo_position == 0L) || (servo_position > 5000L) || (servo_time > 10000L))
        {
            printf("ERR: wrong position or time!!!\n\n");
		    exit(1);
		}
		
        if ((IsFeedback(servo_type) == false) && (argc < 8))
        {
            printf("ERR: wrong number of parameters!!!\n\n");
            exit(1);
        }
        
        if (argc == 8)
        {
            home[channel] = atol(argv[7]);
		    if ((home[channel] == 0L) || (home[channel] > 5000L))
            {
                printf("ERR: wrong home!!!\n\n");
		        exit(1);
		    }
        }
    }
	else //if (servo_demo == SERVO_READ)
	{
		if(argc != 5)
		{
            printf("ERR: wrong number of parameters!!!\n\n");
            exit(1);
		}

        if (IsFeedback(servo_type) == false)
        {
		    printf("ERR: can't read position for non-feedback PWM servos!!!\n\n");
		    exit(1);
        }
    }


	rcservo_SetServo(channel, servo_type);

	if (rcservo_Init(1L << channel) == false)
	{
        printf("ERR: fail to initialize RC Servo lib (%s)!\n", roboio_GetErrMsg());
		return 1;
	}
	
	if (servo_demo == SERVO_PLAY)
	{
        if (home[channel] != 0L)
			rcservo_EnterPlayMode_HOME(home);
		else
			rcservo_EnterPlayMode();

	    printf("moving servo to position %luus...", servo_position);
	    rcservo_MoveOne(channel, servo_position, servo_time);
        printf("completed\n");
	}
	else //if (servo_demo == SERVO_READ)
	{
		rcservo_EnterCaptureMode();
	    servo_position = rcservo_CapOne(channel);

		printf("servo position = %luus\n", servo_position);
	}
	
	rcservo_Close();
	return 0;
}

